@@ -7,6 +7,7 @@
 local io = { popen = io.popen }
 local math = { max = math.max }
 local setmetatable = setmetatable
+local spawn = require("awful.spawn")
 -- }}}
 
 
@@ -16,11 +17,10 @@ local pkg_all = {}
 
 
 -- {{{ Packages widget type
-local function worker(format, warg)
+function pkg_all.async(format, warg, callback)
     if not warg then return end
 
     -- Initialize counters
-    local updates = 0
     local manager = {
         ["Arch"]   = { cmd = "pacman -Qu" },
         ["Arch C"] = { cmd = "checkupdates" },
@@ -33,16 +33,34 @@ local function worker(format, warg)
     }
 
     -- Check if updates are available
-    local _pkg = manager[warg]
-    local f = io.popen(_pkg.cmd)
-
-    for line in f:lines() do
-        updates = updates + 1
+    local function parse(str, skiprows)
+        local size, lines, first = 0, "", skiprows or 0
+        for line in str:gmatch("[^\r\n]+") do
+            if size >= first then
+                lines = lines .. (size == first and "" or "\n") .. line
+            end
+            size = size + 1
+        end
+        size = math.max(size-first, 0)
+        return {size, lines}
     end
-    f:close()
 
-    return {_pkg.sub and math.max(updates-_pkg.sub, 0) or updates}
+    -- Select command
+    local _pkg = manager[warg]
+    spawn.easy_async(_pkg.cmd, function(stdout) callback(parse(stdout, _pkg.sub)) end)
 end
 -- }}}
 
+-- {{{ Packages widget type
+local function worker(format, warg)
+    local ret = nil
+
+    pkg_all.async(format, warg, function(data) ret = data end)
+
+    while ret==nil do end
+    return ret
+end
+-- }}}
+
+
 return setmetatable(pkg_all, { __call = function(_, ...) return worker(...) end })
